import { parse as parse$1 } from 'path';
import { readFile } from 'fs/promises';
import postcss from 'postcss';
import postcssModules from 'postcss-modules';
import sassPostCssPlugin from '@csstools/postcss-sass';
import sassSyntax from 'postcss-scss';
import lessSyntax from 'postcss-less';
import MagicString from 'magic-string';
import { parse } from '@babel/parser';
import _traverse from '@babel/traverse';
import { isStringLiteral, isJSXExpressionContainer, isCallExpression, isObjectExpression, isIdentifier } from '@babel/types';
import { scandir } from '@nodelib/fs.scandir';
import micromatch from 'micromatch';

async function transformStyles(from, code, options = {}) {
  let manifest = {};
  const extensions = getSyntaxExtensions(from);
  const plugins = [];
  if (extensions?.plugin) {
    plugins.push(extensions.plugin);
  }
  const result = await postcss([...plugins, postcssModules({
    ...options,
    getJSON(fileName, json, outputFileName) {
      manifest = json;
      if (options.getJSON && typeof options.getJSON === "function") {
        options.getJSON(fileName, json, outputFileName);
      }
    }
  })]).process(code, {
    from,
    syntax: extensions?.syntax
  });
  return {
    manifest,
    css: result.css
  };
}
function getSyntaxExtensions(fileUrl) {
  if (fileUrl.endsWith(".scss") || fileUrl.endsWith(".sass")) {
    return {
      plugin: sassPostCssPlugin,
      syntax: sassSyntax
    };
  }
  if (fileUrl.endsWith(".less")) {
    return {
      plugin: undefined,
      syntax: lessSyntax
    };
  }
  return null;
}

// https://github.com/babel/babel/issues/13855
// @ts-ignore
const traverse = _traverse.default;
async function transformJSX({
  moduleId,
  styleModuleId,
  originalCode,
  manifest
}) {
  const source = new MagicString(originalCode);
  const ast = parse(originalCode, {
    plugins: ["typescript", "jsx"],
    sourceType: "module"
  });
  source.prepend(`import "${styleModuleId}";\n`);

  /**
   * No need to walk through JSX if manifest is empty.
   */
  if (Object.keys(manifest).length === 0) {
    return generateRollupTransformResult(source, moduleId);
  }
  traverse(ast, {
    JSXAttribute(path) {
      const {
        value
      } = path.node;
      if (isClassAttributePath(path)) {
        /**
         * Handle the simplest case: <div class="foobar" />
         */
        if (isStringLiteral(value)) {
          transformClassStringLiteral(value);
        }

        /**
         * Handle className helper functions, e.g:
         * <div class={ classNames({ foo: true }, ["bar", "baz"]) }/>
         */
        if (isJSXExpressionContainer(value) && isCallExpression(value.expression)) {
          transformClassNamesObject(path);
          transformArrayMemberClassName(path);
          return;
        }
        return;
      }
      if (isClassListAttributePath(path) && isJSXExpressionContainer(value) && isObjectExpression(value.expression)) {
        transformClassNamesObject(path);
      }
    }
  });
  function transformClassStringLiteral(value) {
    const {
      start,
      end
    } = value;
    const classSegments = value.value.split(" ").filter(s => !!s);
    const resolvedClassSegments = classSegments.map(className => {
      return manifest[className] || className;
    });
    const finalClassName = resolvedClassSegments.join(" ");
    source.overwrite(start, end, `"${finalClassName}"`);
  }
  function transformClassNamesObject(path) {
    const node = path.node;
    traverse(node, {
      noScope: true,
      ObjectProperty(prop) {
        if (isIdentifier(prop.node.key)) {
          const {
            start,
            end
          } = prop.node.key;
          const {
            name
          } = prop.node.key;
          source.overwrite(start, end, `"${manifest[name] || name}"`);
          return;
        }
        if (isStringLiteral(prop.node.key)) {
          transformClassStringLiteral(prop.node.key);
        }
      }
    });
  }
  function transformArrayMemberClassName(path) {
    const node = path.node;
    traverse(node, {
      noScope: true,
      ArrayExpression(prop) {
        prop.node.elements.forEach(element => {
          if (isStringLiteral(element)) {
            transformClassStringLiteral(element);
          }
        });
      }
    });
  }
  return generateRollupTransformResult(source, moduleId);
}
function generateRollupTransformResult(source, moduleId) {
  return {
    code: source.toString(),
    map: source.generateMap({
      source: moduleId
    })
  };
}
function isClassAttributePath(path) {
  return path.node.name.name === "class" || path.node.name.name === "className";
}
function isClassListAttributePath(path) {
  return path.node.name.name === "classList";
}

async function resolveStylesFromDirectory({
  componentFilePath,
  directory,
  options
}) {
  const parsedPath = parse$1(componentFilePath);
  const componentName = parsedPath.name;
  const directoryName = parsedPath.dir;
  if (typeof options.resolveStyleForComponent === "function") {
    const result = await options.resolveStyleForComponent(componentName, directoryName, componentFilePath);
    return result ? [result] : [];
  }
  if (typeof options.styleGlob === "string") {
    const matches = micromatch(directory, options.styleGlob, {
      basename: true
    });
    return handleGlobMatches(matches, options.resolveAllStylesAsModules);
  }
  if (!options.styleExtensions) {
    throw new Error("One of 'resolveStyleForComponent', 'styleGlob' or 'styleExtensions' is required");
  }
  const globs = options.styleExtensions.map(ext => {
    /**
     * If `matchComponentName` is enabled then the result glob
     * would look like this: `component*.{css|scss|less}`.
     * Otherwise: `*.{css|scss|less}` (matches any style file)
     */
    const prefix = options.matchComponentName ? componentName : "";
    return `${prefix}*${ext}`;
  });
  const matches = micromatch(directory, globs, {
    basename: true
  });
  return handleGlobMatches(matches, options.resolveAllStylesAsModules);
}
function handleGlobMatches(matches, resolveAllStylesAsModules = false) {
  return matches.map(filePath => {
    const parsedStylePath = parse$1(filePath);
    let isModule;
    if (resolveAllStylesAsModules) {
      isModule = true;
    } else {
      isModule = parsedStylePath.name.endsWith(".module");
    }
    return {
      filePath,
      isModule
    };
  });
}
async function getAllFilesInDirectory(dir) {
  return new Promise((resolve, reject) => {
    scandir(dir, (err, result) => {
      if (err) {
        return reject(err);
      }
      const paths = result.filter(entry => !entry.stats?.isDirectory()).map(entry => entry.path);
      resolve(paths);
    });
  });
}

const defaultOptions = {
  componentExtensions: [".tsx", ".jsx"],
  styleExtensions: [".css", ".scss", ".less"],
  matchComponentName: true
};
const virtualModuleIdPrefix = "virtual:css-auto-import";
const resolvedVirtualModuleIdPrefix = `\0${virtualModuleIdPrefix}`;
const virtualModuleUrlSeparator = "originalModuleId=";
function cssAutoImport(userOptions = {}) {
  const options = {
    ...defaultOptions,
    ...userOptions
  };
  const componentToCssMap = new Map();
  const styleModuleIdToCssMap = new Map();

  /**
   * Used in development mode for watching
   */
  const directoryComponentsMap = new Map();
  const styleModuleIdToComponentIdsMap = new Map();
  let modulesOptions;
  async function shouldTransformModule(id) {
    const path = parse$1(id);
    const fileName = path.base;
    if (typeof options.shouldTransformComponent === "function") {
      return await options.shouldTransformComponent(fileName, id);
    }
    if (!options.componentExtensions) {
      throw new Error("Either 'shouldTransformComponent' or 'componentExtensions' is required");
    }

    /**
     * TODO: add pragma to disable transform inside component file
     */
    return options.componentExtensions.includes(path.ext);
  }

  /**
   * Generates a URL that looks like the following:
   * virtual:css-auto-import/e2fdd67cf4194/originalModuleId=<path> 
   */
  function generateVirtualStyleModuleId(moduleId) {
    const uniqueId = Math.random().toString(16).slice(2);
    return `${virtualModuleIdPrefix}/${uniqueId}/${virtualModuleUrlSeparator}${moduleId}`;
  }
  function addReferenceRecord(map, key, moduleId) {
    let dependentModules = map.get(key);
    if (!dependentModules) {
      dependentModules = new Set();
      map.set(key, dependentModules);
    }
    dependentModules.add(moduleId);
  }
  return {
    enforce: "pre",
    name: "css-auto-import",
    configResolved(config) {
      if (config.css?.modules && typeof config.css.modules === "object") {
        modulesOptions = config.css.modules;
      }
    },
    configureServer(server) {
      const reloadModuleById = id => {
        const module = server?.moduleGraph.getModuleById(id);
        if (module) {
          server?.reloadModule(module);
        }
      };
      server.watcher.on("add", fileId => {
        const path = parse$1(fileId);
        const dir = path.dir;
        const dependentModules = directoryComponentsMap.get(dir);
        /**
         * TODO: check if fileId is a style module
         */
        dependentModules?.forEach(reloadModuleById);
      });
      server.watcher.on("unlink", fileId => {
        if (styleModuleIdToComponentIdsMap.has(fileId)) {
          const dependentModules = styleModuleIdToComponentIdsMap.get(fileId);
          dependentModules?.forEach(reloadModuleById);
          styleModuleIdToComponentIdsMap.delete(fileId);
        }
      });
      server.watcher.on("unlinkDir", dir => {
        if (directoryComponentsMap.has(dir)) {
          /**
           * TODO: check whether need to clean other maps
           */
          directoryComponentsMap.delete(dir);
        }
      });
      server.watcher.on("change", fileId => {
        if (!styleModuleIdToComponentIdsMap.has(fileId)) {
          return;
        }
        const dependentModules = styleModuleIdToComponentIdsMap.get(fileId);
        dependentModules?.forEach(reloadModuleById);
      });
    },
    resolveId(id) {
      if (!id.startsWith(virtualModuleIdPrefix)) {
        return undefined;
      }
      return "\0" + id;
    },
    load(id) {
      if (!id.startsWith(resolvedVirtualModuleIdPrefix)) {
        return undefined;
      }
      const separatorIdx = id.indexOf(virtualModuleUrlSeparator);
      /**
       * Extract the part that goes after the `originalModuleId=`
       */
      const referencedModuleId = id.substring(separatorIdx + virtualModuleUrlSeparator.length);
      if (!styleModuleIdToCssMap.has(referencedModuleId)) {
        throw new Error(`Failed to resolve CSS module ${referencedModuleId}`);
      }
      return styleModuleIdToCssMap.get(referencedModuleId);
    },
    async transform(code, id) {
      if (!(await shouldTransformModule(id))) {
        return null;
      }
      const parsedModuleId = parse$1(id);
      const componentDirectory = parsedModuleId.dir;
      const directory = await getAllFilesInDirectory(componentDirectory);
      const styleModules = await resolveStylesFromDirectory({
        componentFilePath: id,
        directory,
        options
      });
      if (!styleModules || !styleModules.length) {
        return null;
      }

      /**
       * TODO: consider multiple style modules support
       */
      if (styleModules.length > 1) {
        this.warn(`Found multiple style module matches for ${id}. Using the first match: ${styleModules[0].filePath}`);
      }
      const [styleModule] = styleModules;
      const {
        filePath: styleModuleId,
        isModule
      } = styleModule;
      const styleModuleCode = (await readFile(styleModuleId)).toString();
      let css;
      let manifest = {};
      if (isModule) {
        try {
          const result = await transformStyles(styleModuleId, styleModuleCode, modulesOptions);
          css = result.css;
          manifest = result.manifest;
        } catch (err) {
          this.error({
            message: err.reason,
            loc: {
              column: err.column,
              line: err.line,
              file: err.file
            }
          });
        }
      } else {
        /**
         * No need to transform non-module styles. Just use original CSS as is.
         */
        css = styleModuleCode;
      }
      if (css === undefined) {
        return null;
      }
      const normalizedStyleModuleId = styleModuleId.replace(".module", "");
      const styleVirtualModuleId = generateVirtualStyleModuleId(normalizedStyleModuleId);
      componentToCssMap.set(id, css);
      styleModuleIdToCssMap.set(normalizedStyleModuleId, css);
      addReferenceRecord(styleModuleIdToComponentIdsMap, styleModuleId, id);
      addReferenceRecord(directoryComponentsMap, componentDirectory, id);
      return transformJSX({
        moduleId: id,
        styleModuleId: styleVirtualModuleId,
        originalCode: code,
        manifest
      });
    }
  };
}

export { cssAutoImport as default };
//# sourceMappingURL=index.mjs.map
